local classNamesM = {}
local classNamesF = {}
FillLocalizedClassList(classNamesM ,false)
FillLocalizedClassList(classNamesF ,true)

local subbedChannels = {} -- chat windows that are subbed to show /f3ptwink

local HEX_CLASS_COLORS = {  --saves having to convert them from the r,g,b values bliz uses
	["DEATHKNIGHT"] = "C41F3B",
	["DRUID"] = "FF7D0A",
	["HUNTER"] = "ABD473",
	["MAGE"] = "69CCF0",
	["MONK"] = "00FF96",
	["PALADIN"] = "F58CBA",
	["PRIEST"] = "FFFFFF",
	["ROGUE"] = "FFF569",
	["SHAMAN"] = "0070DE",
	["WARLOCK"] = "9482C9",
	["WARRIOR"] = "C79C6E",
}

local wowSendChatMessage = SendChatMessage;  --Backup the existing version of SendChatMessage, so it can be called as part of our own version.

function F3PChat_SendChatMessage(msg, chatType, ...)  --Create the hooking fuction
	local a = {...}
	--if the message is being sent to our custom channel, you aren't capable of sending normally (not on P2P), and isn't for another addon.
	if ( (chatType == "CHANNEL") and (a[#a] == GetChannelName(F3PAddonGlobalVars.Channel1Name))) and (not string.match(msg, "^f3p=")) and (IsTrialAccount() or F3PAddon_Variables.Settings.TrialFix) then
		F3PChat_RouteChatMessage(msg)  --send via addon
		F3PChat_FormatIncoming(msg, F3PAddonGlobalVars.myName) --print sent message
	else  --otherwise send as normal
		wowSendChatMessage(msg, chatType, ...)
	end
end

SendChatMessage = F3PChat_SendChatMessage  --Hook WoW's SendChatMessage, so our version gets called first.

function F3PChat_RouteChatMessage(msg)
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, class, loc, connected, status = GetFriendInfo(y)
		if connected and name then
			ChatThrottleLib:SendAddonMessage("NORMAL", F3PAddonGlobalVars.AddonChannel1Name, msg, "WHISPER", name)
		end
	end	
end

function F3PChat_FilterP2PTrolls(self, event, ...)  --filters messages sent to the channel by p2ps, or as a whisper from anyone
	local msg, sender, _, _, _, _, _, _, chanName, _, _, _ = ...
	if (event == "CHAT_MSG_CHANNEL") and (chanName == F3PAddonGlobalVars.Channel1Name) and (F3PChat_IsBlocked(sender)) then
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_FilterP2PTrolls - "..sender.."'s message to the channel was blocked.") end
		return true
	elseif (event == "CHAT_MSG_WHISPER") and (F3PChat_IsBlocked(sender)) then
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_FilterP2PTrolls - "..sender.."'s whisper was blocked.") end
		return true
    else
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_FilterP2PTrolls - "..sender.."'s message was allowed.") end
		return false
	end
end

function F3PChat_FormatIncoming(msg, sender)
	if F3PChat_IsBlocked(sender) and (sender ~= F3PAddonGlobalVars.myName) then  --filters messages to the channel that come in via the addon-addon channel
		return
	end
	
	local senderClassColor = F3PChat_GetSenderClassColorAsHex(sender)
	local channelNumber = F3PChat_GetChannelNumber(F3PAddonGlobalVars.Channel1Name)
	local channelColor = F3PChat_GetChannelColor(channelNumber)
	local chanHeader = channelNumber..". "..F3PAddonGlobalVars.Channel1Name
	if F3PAddonGlobalVars.channelHeader then
		chanHeader = F3PAddonGlobalVars.channelHeader
	end
	local header = "|cFF"..channelColor.."["..chanHeader.."] [|cFF"..senderClassColor.."|Hplayer:"..sender.."|h"..sender.."|h|cFF"..channelColor.."] "
	msg = msg:gsub("%[(MogIt[^%]]+)%]","|cFFCC99FF|H%1|h[MogIt]|h|r");
	local body = string.gsub(msg,"|r","|cFF"..channelColor) --fixes item links reverting channel color back to white
	local subbedCheck = 0
	for i,v in ipairs(F3PChat_GetSubscribedChannels(F3PAddonGlobalVars.Channel1Name)) do
		getglobal("ChatFrame"..v):AddMessage(header..body)
		subbedCheck = 1
	end
	if subbedCheck == 0 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FF" .. string.format(F3PLoc.chat["NO_SUBBED_CHAT_WINDOW_1"], F3PAddonGlobalVars.Channel1Name) ..".|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FF" .. string.format(F3PLoc.chat["NO_SUBBED_CHAT_WINDOW_2"], F3PAddonGlobalVars.Channel1Name) ..".|r")
	end
end

--returns the chat frame numbers that are subscribed to the channel
function F3PChat_GetSubscribedChannels(cName)
	local subbedChannels = {}
	for i = 1,NUM_CHAT_WINDOWS do
		if tContains({GetChatWindowChannels(i)}, cName) then
			tinsert(subbedChannels, i)
		end
	end
	return subbedChannels
end

function F3PChat_GetChannelNumber(channelName)
	return GetChannelName(channelName)
end

function F3PChat_GetSenderClassColorAsHex(sender)
	if (sender == F3PAddonGlobalVars.myName) then
		local lname, name = UnitClass("player")
		return HEX_CLASS_COLORS[name]
	end
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, localizedClass, loc, connected, status = GetFriendInfo(y)
		if (sender == name) then
			for n, v in pairs(classNamesM) do
				if (v == localizedClass) then
					return HEX_CLASS_COLORS[n]
				end
			end
			for n, v in pairs(classNamesF) do
				if v == localizedClass then
					return HEX_CLASS_COLORS[n]
				end
			end
		end
	end
	return "808080"
end


--returns the hex value color of the channel
function F3PChat_GetChannelColor(cNumber)
	local r, g, b
	for i, v in pairs({ChatTypeInfo["CHANNEL"..cNumber]}) do
		for x, y in pairs(v) do
			if x == "r" then
				r = y
			elseif x == "g" then
				g = y
			elseif x == "b" then
				b = y
			end
		end
	end
	return F3PChat_RGBToHex(r, g, b)
end	

function F3PChat_RGBToHex(r, g, b)
   local r = r <= 1 and r >= 0 and r or 1
   local g = g <= 1 and g >= 0 and g or 1
   local b = b <= 1 and b >= 0 and b or 1
   return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

function F3PChat_IsBlocked(sender)
	local senderLevel
	
	--if the sending character has a level entry in the database
	if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["level"] then
		--if they've been flagged as having been banned
		if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["banned"]) then
			--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.." has been banned from the channel") end
			return true
		--if their level is OK
		elseif F3PChat_LevelOK(tonumber(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["level"])) then
			--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.."'s database entry has an acceptable level: "..tostring(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["level"])) end
			return false
		--if their level is not OK
		elseif not F3PChat_LevelOK(tonumber(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["level"])) and (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender]["level"])  then
			local alts = F3PData_ConfirmedAlts(sender)
			for x = 1, #alts do
				if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][alts[x]] then
					--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - checking alt "..alts[x]) end
					--check for banned alts (not yet implemented)
					if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][alts[x]]["banned"]) then
						--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.." has had an alt banned from the channel") end
						return true					
					--check for a confirmed L20 alt
					elseif (tonumber(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][alts[x]]["level"]) == 20) then
						--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.." had a level 20 alt") end
						return false
					end
				end
			end
			--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.."'s database entry has an unacceptable level: "..tostring(senderLevel)) end
			return true
		end
	end
	
	--otherwise fall back to the friends list
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, class, loc, connected, status = GetFriendInfo(y)
		if name == sender then
			senderLevel = tonumber(level)
			--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.."'s friends list entry has level: "..tostring(senderLevel)) end
		end
	end
	if not senderLevel then
		senderLevel = 0
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - No level found for "..sender) end
	end
	if F3PChat_LevelOK(senderLevel) then
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.." was of an acceptable level") end
		return false
	end
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("F3PChat_IsBlocked - "..sender.." was of an unknown level") end
	return false
end

function F3PChat_LevelOK(playerLevel)
	if (playerLevel < 3) and (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction].Settings.BlockLowLevel == 1) then
		return false
	elseif (playerLevel == 24) and (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction].Settings.BlockTwentFour == 1) then
		return false
	else
		return true
	end
end






















