
--#############################################################
--# Form data packets containing this character's information #
--#############################################################

function F3PData_OutgoingDataPacket()
	
	local _, class = UnitClass(F3PAddonGlobalVars.myName)
	c1, c2 = string.match(class, "^(.)(.+)$")
	class = c1..string.lower(c2)
	
	local level = UnitLevel(F3PAddonGlobalVars.myName)

	local _, race = UnitRace(F3PAddonGlobalVars.myName)
	if race == "Scourge" then race = "Forsaken" end
	if race == "BloodElf" then race = "Blood Elf" end
	
	local gender = UnitSex(F3PAddonGlobalVars.myName)
	if gender == 3 then
		gender = "female"
	elseif gender == 2 then
		gender = "male"
	end

	local spec = GetSpecialization()

	local trial
	if IsTrialAccount() then
		trial = "1"
	else
		trial = "0"
	end
		
	local xpLocked = IsXPUserDisabled()
	if xpLocked then
		xplocked = "1"
	else
		xpLocked = "0"
	end

	local data = {
		["class"] = class,
		["level"] = level,
		["race"] = race,
		["gender"] = gender,
		["spec"] = spec,
		["trial"] = trial,
		["xpLocked"] = xpLocked,
		["version"] = F3PAddonGlobalVars.addonVersion,
	}

	local dataMsg = ""
	
	for i, v in pairs(data) do
		dataMsg = dataMsg..i..","..tostring(v)..":"
	end

	return dataMsg
end

function F3PData_OutgoingAltsPacket()
	local altsMsg = "f3p=dataalts:"
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("My alts are:") end
	--[[]]if F3PAddon_Variables.Settings.Verbose then print(table.concat(F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm], ",")) end
	altsMsg = altsMsg..table.concat(F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm], ",")
	return altsMsg
end

--#########################################
--# Send data packets to other characters #
--#########################################

function F3PData_SendMyData_ToAll()
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("Sending to everyone: ".."f3p=datadall:"..F3PData_OutgoingDataPacket()) end
	F3PData_RouteDataPacket("f3p=datadall:"..F3PData_OutgoingDataPacket())
	if (#F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm] > 1) then --if this account has alts
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("Sending to everyone: "..F3PData_OutgoingAltsPacket()) end
		F3PData_RouteDataPacket(F3PData_OutgoingAltsPacket())
	end
end

function F3PData_SendMyData_ToOne(sender)
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("Sending to "..sender..": ".."f3p=datadone:"..F3PData_OutgoingDataPacket()) end
	ChatThrottleLib:SendAddonMessage("NORMAL", F3PAddonGlobalVars.AddonChannel1Name, "f3p=datadone:"..F3PData_OutgoingDataPacket(), "WHISPER", sender)
	if (#F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm] > 1) then --if this account has alts
		--[[]]if F3PAddon_Variables.Settings.Verbose then print("Sending to "..sender..": "..F3PData_OutgoingAltsPacket()) end
		ChatThrottleLib:SendAddonMessage("NORMAL", F3PAddonGlobalVars.AddonChannel1Name, F3PData_OutgoingAltsPacket(), "WHISPER", sender)
	end
end

--#########################
--# Data routing function #
--#########################

---[[
function F3PData_RouteDataPacket(msg)
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, class, loc, connected, status = GetFriendInfo(y)
		if connected and name then
			--[[]]if F3PAddon_Variables.Settings.Verbose then print("sending data packet to "..name) end
			ChatThrottleLib:SendAddonMessage("NORMAL", F3PAddonGlobalVars.AddonChannel1Name, msg, "WHISPER", name)
		end
	end	
end
--]]

--####################################################################
--# Special case to have everyone request your level from the server #
--####################################################################

function F3PData_IDinged()
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("sending ding announcement") end
	F3PData_RouteDataPacket("f3p=ding")
end

function F3PData_RequestWhoLevel(sender)
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("received ding announcement from "..sender..", adding name to whoList and requesting /who.") end
	SetWhoToUI(0)
	SendWho('n-"'..sender..'"')
	tinsert(F3PAddonGlobalVars.whoList, sender)
end

function F3PData_ReceiveWhoLevel()
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("Who received.") end
	local numResults, totalCount = GetNumWhoResults()
	for x = 1, numResults do
		local name, guild, level, race, class, zone, classFileName, sex = GetWhoInfo(x)
		if tContains(F3PAddonGlobalVars.whoList, name) then
			if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][name] then
				--[[]]if F3PAddon_Variables.Settings.Verbose then print("Updating database entry for "..name.." with level "..level) end
				F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][name]["level"] = level
				for y = 1, #F3PAddonGlobalVars.whoList do
					if F3PAddonGlobalVars.whoList[y] == name then
						tremove(F3PAddonGlobalVars.whoList, y)
					end
				end
			end
		end
	end
end



--#####################################
--# Add incoming data to the database #
--#####################################

function F3PData_Data_Add(...)
	local input = ...
	dataName = string.match(input, ":([^:]+)$")

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][dataName] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][dataName] = {}
	end
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("------------") end
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("Adding data for "..dataName..":") end

	
	for i, v in string.gmatch(input, ":([^,]+),([^:]+)") do
		--[[]]if F3PAddon_Variables.Settings.Verbose then print(i..", "..v) end
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][dataName][i] = v
	end	
end

function F3PData_Alts_Add(sender, input)

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender] = {}
	end
	
	local output = string.match(input, "^f3p=dataalts:([^:]+)$")
	
	--[[]]if F3PAddon_Variables.Settings.Verbose then print("alts of "..sender.." are: "..output) end
	F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][sender].Alts = output
end

--##########################
--# Log out time recording #
--##########################


function F3PData_LLIT_Single(...)	--Last Logged In Time
	local a,b,c,d,e,f,g,h,i = ...
	currentTime = date("y%yM%md%dh%Hm%M")
	currentTime = string.gsub(currentTime,"h%d%d", "h"..string.format("%02d", GetGameTime()))
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][b] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][b] = {}
	end
	F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][b]["LLIT"] = currentTime
end

function F3PData_LLIT_Group()	--Last Logged In Time
	currentTime = date("y%yM%md%dh%Hm%M")
	currentTime = string.gsub(currentTime,"h%d%d", "h"..string.format("%02d", GetGameTime()))
	for i, v in pairs(F3PAddonGlobalVars.onlineFriendsList) do
		if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][v] == nil) then
			F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][v] = {}
		end
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][v]["LLIT"] = currentTime
	end
end


--#########################################
--# show which alts belong to a character #
--#########################################

function F3PData_ConfirmedAlts(name)
	local confirmedList = {}
	if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][name] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][name].Alts then
		for n in string.gmatch(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][name].Alts, "([^,]+)") do
			if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][n] and F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][n].Alts then
				if string.match(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][n].Alts, name) then
					tinsert(confirmedList, n)
				end
			end
		end
	end
	return confirmedList
end

--###########################
--# Compare version numbers #
--###########################

function F3PData_GetIfOldVersion()
	local isOld = 0
	local ourVersion = F3PAddonGlobalVars.addonVersion
	local otherVersion
	for i, v in pairs(F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"]) do
		if F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][i].version then
			otherVersion = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][i]["version"]
			if F3PData_CompareVersions(ourVersion, otherVersion) then
				isOld = 1
				ourVersion = otherVersion
			end
		end
	end
	if isOld == 1 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FF" .. string.format(F3PLoc.data["OUTDATED"], ourVersion))
	end
end

function F3PData_CompareVersions(ours, others)

	local a1 = string.match(ours, "^([%d]+)\.")
	local a2 = string.match(ours, "\.([%d]+)\.")
	local a3 = string.match(ours, "[%d]+\.[%d]+\.([^b]+)b?$")

	local b1 = string.match(others, "^([%d]+)\.")
	local b2 = string.match(others, "\.([%d]+)\.")
	local b3 = string.match(others, "[%d]+\.[%d]+\.([^b]+)b?$")

	if tonumber(a1) < tonumber(b1) then
		return true
	elseif tonumber(a1) == tonumber(b1) then
		if tonumber(a2) < tonumber(b2) then
			return true
		elseif tonumber(a2) == tonumber(b2) then
			if tonumber(a3) < tonumber(b3) then
				return true
			elseif tonumber(a3) == tonumber(b3) then
				return false
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end
	
end

--###################################
--# Set up basic database structure #
--###################################

function F3PData_BuildDBSkeleton()
	if (F3PAddon_Variables == nil) or (F3PAddon_Variables.Version == nil) then --if the complete table is blank, as when the addon is first installed, or the database predates versioning
		F3PAddon_Variables = {			--make this it's basic structure
			["Version"] = F3PAddonGlobalVars.addonVersion,
			["Settings"] = {
				["Verbose"] = F3PAddonGlobalVars.verbose,
				["TrialFix"] = F3PAddonGlobalVars.stillTreatedAsTrialFix,
			},
			["Alts"] = {
				[F3PAddonGlobalVars.thisRealm] = {},
			},
			["Realms"] = {
				[F3PAddonGlobalVars.thisRealm] = {
					[F3PAddonGlobalVars.myFaction] = {
						["Characters"] = {},
						["Settings"] = {
							["Channel"] = F3PAddonGlobalVars.DefaultChannelName,
						},
					},
				},
			},
		}
	end
	
	if (F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm] == nil) then --if the Alts data for this realm doesn't exist
		F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm] = {}		 --add a new blank table for it
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm] == nil) then --if the data for this realm doesn't exist,
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm] = {}		 --add a new blank table for it
	end

	if (F3PAddon_Variables.Settings.Verbose == nil) then
		F3PAddon_Variables.Settings.Verbose = false
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction] == nil) then --if the data for this faction doesn't exist for this realm,
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction] = {}		 --add a new blank table for it
	end

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"] = {} -- make a new sub table for the character names
	end

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"] = {} --set it to the default
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["Channel"] == nil) then --if there's no channel set for this realm/faction
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["Channel"] = F3PAddonGlobalVars.DefaultChannelName --set it to the default
	end

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockLowLevel"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockLowLevel"] = F3PAddonGlobalVars.blockLowLevel --set it to the default
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockTwentyFour"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockTwentyFour"] = F3PAddonGlobalVars.blockTwentyFour --set it to the default
	end

	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockGuild"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["BlockGuild"] = F3PAddonGlobalVars.blockGuild --set it to the default
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["ShowCharacterTooltips"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["ShowCharacterTooltips"] = F3PAddonGlobalVars.showCharacterTooltips --set it to the default
	end
	
	if (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["ChannelWindows"] == nil) then
		F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Settings"]["ChannelWindows"] = F3PAddonGlobalVars.DefaultChannelWindows --set it to the default
	end	

	if not tContains(F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm], F3PAddonGlobalVars.myName) then
		tinsert(F3PAddon_Variables.Alts[F3PAddonGlobalVars.thisRealm], F3PAddonGlobalVars.myName)
	end
	
end


--If changes to the addon also require a change to the database structure, put code here to make those changes (or wipe the DB)
--then update the DB version to match the current addon version
function F3PData_NewVersionDatabase()
	DBVersion = F3PAddon_Variables["Version"]
	if (F3PAddonGlobalVars.addonVersion ~= DBVersion) then
		if DBVersion == "1.2.10b" then
			--fix stuff
			local DB = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"]
			--specs changed in mop, so each class has specs numbered 1-3 (4 for druids), instead of the old system where each spec had it's own code.
			--also since cata characters under level 10 can no longer pick a specialisation.
			--this wipes the old version specs from the database.
			print("|cFFFF00FF[F3PAddon] cleaning up the database to keep it compatable with the new addon version.")
			local under10s, over10druids, over10s, noversion = 0, 0, 0, 0
			for i, v in pairs(DB) do
				--Clean up specs that aren't within the 1-3 range (1-4 for druids), or are on characters under level 10.
				if DB[i]["level"] then
					if tonumber(DB[i]["level"]) < 10 then
						if DB[i]["spec"] then
							DB[i]["spec"] = nil
							under10s = under10s + 1
						end
					elseif tonumber(DB[i]["level"]) > 10 then
						if DB[i]["spec"] and DB[i]["class"] then
							if DB[i]["class"] == "Druid" and tonumber(DB[i]["spec"]) > 4 then
								DB[i]["spec"] = nil
								over10druids = over10druids + 1
							elseif DB[i]["class"] ~= "Druid" and tonumber(DB[i]["spec"]) > 3 then
								DB[i]["spec"] = nil
								over10s = over10s + 1
							end
						end
					end
				end
				--Remove entries that have no version number listed
				if not DB[i]["version"] then
					DB[i] = nil
					noversion = noversion + 1
				end
			end

			print("|cFFFF00FF"..under10s.." under 10s, "..over10druids.." over 10 druids, and "..over10s.." other over 10s have had their spec wiped from the addon's database, due to having pre MoP values.|r")
			print("|cFFFF00FF"..noversion.." entries were removed due to not having an addon version listed.|r")

		elseif DBVersion == "1.2.11" then
			print("|cFFFF00FF[F3PAddon] Database is compatable with this addon version, so no changes are necessary.")
			F3PData_BuildDBSkeleton()
		else
			print("|cFFFF00FF[F3PAddon] Database structure has changed excessively between versions, and cannot be rebuilt, so it is being wiped and reset to an empty state compatable with the current addon version.")
			--wipe the DB and rebuild
			F3PAddon_Variables = nil
			F3PData_BuildDBSkeleton()
		end
		F3PAddon_Variables.Version = F3PAddonGlobalVars.addonVersion
	end
end






