local specTable = {
	["Deathknight"] = {	"Blood", "Frost", "Unholy" },
	["Druid"] = { "Balance", "Feral", "Guardian", "Restoration"	},
	["Hunter"] = { "Beast Mastery", "Marksmanship", "Survival" },
	["Mage"] = { "Arcane", "Fire", "Frost" },
	["Monk"] = { "Brewmaster", "Mistweaver", "Windwalker" },
	["Paladin"] = {	"Holy",	"Protection", "Retribution" },
	["Priest"] = { "Discipline", "Holy", "Shadow" },
	["Rogue"] = { "Assassination", "Combat", "Subtlety"	},
	["Shaman"] = { "Elemental", "Enhancement", "Restoration" },
	["Warlock"] = {	"Affliction", "Demonology",	"Destruction" },
	["Warrior"] = {	"Arms",	"Fury",	"Protection" },
}

local classNamesM = {}
local classNamesF = {}
FillLocalizedClassList(classNamesM ,false)
FillLocalizedClassList(classNamesF ,true)

local HEX_CLASS_COLORS = {  --saves having to convert them from the r,g,b values bliz uses
	["Deathknight"] = "C41F3B",
	["Druid"] = "FF7D0A",
	["Hunter"] = "ABD473",
	["Mage"] = "69CCF0",
	["Monk"] = "00FF96",
	["Paladin"] = "F58CBA",
	["Priest"] = "FFFFFF",
	["Rogue"] = "FFF569",
	["Shaman"] = "0070DE",
	["Warlock"] = "9482C9",
	["Warrior"] = "C79C6E",
}


local function showTooltip(...)
	local frame, linkdata, link = ...

	if string.match(linkdata, "^player:") and (F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction].Settings.ShowCharacterTooltips == 1) then
		local playerName = string.match(linkdata, "player:([^:$]+)")
		local DB = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][playerName]
		if DB then --excludes all players that aren't part of the database
			ShowUIPanel(GameTooltip)
			GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText(F3PInfo_GetTooltipContents(playerName))
			GameTooltip:Show()
		end
	end
end

local function hideTooltip(...)
    HideUIPanel(GameTooltip)
end
	

local function setOrHookHandler(frame, script, func)
    if frame:GetScript(script) then
        frame:HookScript(script, func)
    else
        frame:SetScript(script,func)
	end
end

for v = 1, NUM_CHAT_WINDOWS do
	local frame = getglobal("ChatFrame"..v)
	if frame then
		setOrHookHandler(frame, "OnHyperLinkEnter", showTooltip)
		setOrHookHandler(frame, "OnHyperLinkLeave", hideTooltip)
    end
end


function F3PInfo_GetTooltipContents(playerName)  --constructs and returns the text string that will be shown in the tooltip.
	local DB = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][playerName]
	if DB["class"] or DB["level"] or DB["gender"] or DB["race"] then 
		local playerString = F3PInfo_GetFormattedPlayerData(playerName)
		local altsString = ""
		if DB["Alts"] then
			local playerAlts = F3PData_ConfirmedAlts(playerName)


			local toRemove = nil  --strip out the player's name from the list of alts
			for i,v in ipairs(playerAlts) do
				if v == playerName then
					toRemove = i
				end
			end
			if toRemove then
				table.remove(playerAlts, toRemove)
			end
			
			if #playerAlts > 0 then  --if there's still alts start building the text
				altsString = "Alts:\n\n"
				for i,v in ipairs(playerAlts) do
					--print(v)
					altsString = altsString..F3PInfo_GetFormattedPlayerData(v)
				end
				altsString = altsString.."\n"
			end
		end
		
		local xpLock, accountType, addonVersion = "", "", ""
		if DB["xpLocked"] == "1" then xpLock = "XP locked " end
		if DB["trial"] == "1" then accountType = "F3P account" elseif DB["trial"] == "0" then accountType = "P2P account" end
		if DB["version"] then addonVersion = "Player's addon version is "..DB["version"].."." end
	
		local footer = xpLock..accountType..".  "..addonVersion
		
		return "|cFFCCCCCC"..playerString.."\n"..altsString..footer.."|r"
	else
		return playerName.." is either not running the addon, or has an obsolete version."
	end
end

function F3PInfo_GetFormattedPlayerData(playerName)
	local DB = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][playerName]
	
	local playerLevel, playerGender, playerRace, playerClass, playerSpec = "", "", "", "", ""
	if DB["level"] then playerLevel = "Level "..DB["level"].." " end
	if DB["gender"] then playerGender = DB["gender"].." " end
	if DB["race"] then playerRace = DB["race"].." " end
	if playerRace == "BloodElf " then playerRace = "Blood Elf " end
	if DB["class"] then playerClass = DB["class"] end
	if DB["spec"] and DB["class"] then playerSpec = specTable[DB["class"]][tonumber(DB["spec"])] end
	if not playerSpec then playerSpec = "" elseif playerSpec ~= "" then playerSpec = playerSpec.." " end

	local playerString1 = "|r|cFF"..F3PInfo_GetCharacterClassColorAsHex(playerName)..playerName.."|r = "..playerLevel
	local playerString2 = string.lower(playerGender..playerRace..playerSpec..playerClass..".")

	return playerString1..playerString2.."\n"
end


function F3PInfo_GetCharacterClassColorAsHex(character)
	local n = F3PAddon_Variables.Realms[F3PAddonGlobalVars.thisRealm][F3PAddonGlobalVars.myFaction]["Characters"][character]["class"]
	if n then
		return HEX_CLASS_COLORS[n]
	end
	return "808080"
end