local WSGStatus
local ABStatus
local lastWSGStatus = "none"
local lastABStatus = "none"


function F3PQ_SendUpdate()
	msg = nil
	WSGStatus = "none"
	ABStatus = "none"
	for i = 1, GetMaxBattlefieldID() do
		status, mapName = GetBattlefieldStatus(i)
		if mapName == F3PLoc.q["WARSONG_GULCH"] then
			WSGStatus = status
		elseif mapName == F3PLoc.q["ARATHI_BASIN"] then
			ABStatus = status
		end
	end

	--as the server spam status events regardless of change, check to only send updates if it's actually changed
	if (WSGStatus ~= "none" and WSGStatus ~= lastWSGStatus) then
		msg = "W"..WSGStatus
	elseif (WSGStatus == "none" and lastWSGStatus ~= "none") then
		msg = "Wnone"
	end

	if (ABStatus ~= "none" and ABStatus ~= lastABStatus) then
		msg = "A"..ABStatus
	elseif (ABStatus == "none" and lastABStatus ~= "none") then
		msg = "Anone"
	end

	if (WSGStatus == "active") or (ABStatus == "active") then
		F3PAddonQueueInfoFrame:Hide()
	end
	
	if msg then
		local f_num = GetNumFriends()
		for y = 1, f_num do
			local name, level, class, loc, connected, status = GetFriendInfo(y)
			if connected and name then
				ChatThrottleLib:SendAddonMessage("NORMAL", "f3pq", msg, "WHISPER", name)
			end
		end
	end
	lastWSGStatus = WSGStatus
	lastABStatus = ABStatus
end

function F3PQ_ReceiveUpdate(...)
	local sender, msg = ...
	local state
	local map

	state = string.match(msg, "^.([%a]+)")

	if state == "none" then
		state = 0
	elseif state == "queued" then
		state = 1
	elseif state == "confirm" then
		state = 2
	elseif state == "active" then
		state = 0
	end
	
	map = string.match(msg, "^(.)")

	--update table
	if not F3PAddonGlobalVars.queueData[sender] then
		F3PAddonGlobalVars.queueData[sender] = {}
		F3PAddonGlobalVars.queueData[sender]["class"] = F3PChat_GetSenderClassColorAsHex(sender)
	end
	F3PAddonGlobalVars.queueData[sender][map] = state

	F3PQ_TableToDisplay()
end

function F3PQ_TableToDisplay()
	--parse table and if both maps for a name have no state remove the name
	local toRemove = {}
	for i, v in pairs(F3PAddonGlobalVars.queueData) do
		if (F3PAddonGlobalVars.queueData[i]["W"] == 0 or F3PAddonGlobalVars.queueData[i]["W"] == nil) and (F3PAddonGlobalVars.queueData[i]["A"] == 0 or F3PAddonGlobalVars.queueData[i]["A"] == nil) then
			tinsert(toRemove, i)
		end
	end
	for x = 1, #toRemove do
		F3PAddonGlobalVars.queueData[toRemove[x]] = nil
	end
	
	--parse table and convert contents to text
	local nameList = {}
	local appendText
	local outputText = "\124cFFFFFFFF" .. F3PLoc.q["OUTPUT_TEXT"]
	
	--create a sorted list of names from the table
	for i, v in pairs(F3PAddonGlobalVars.queueData) do
		tinsert(nameList, i)
	end
	
	table.sort(nameList)
	
	for x = 1, #nameList do
		if F3PAddonGlobalVars.queueData[nameList[x]]["W"] == 1 then
			wState = "\124cFFFF0000" .. F3PLoc.q["QUEUED"]
		elseif F3PAddonGlobalVars.queueData[nameList[x]]["W"] == 2  then
			wState = "\124cFF00FF00" .. F3PLoc.q["POPPED"]
		else
			wState = "      "
		end
		
		if F3PAddonGlobalVars.queueData[nameList[x]]["A"] == 1 then
			aState = "\124cFFFF0000" .. F3PLoc.q["QUEUED"]
		elseif F3PAddonGlobalVars.queueData[nameList[x]]["A"] == 2  then
			aState = "\124cFF00FF00" .. F3PLoc.q["POPPED"]
		else
			aState = ""
		end
		
		--fix for regional characters that are considered to have a length of 2 while displayed as 1
		local nameLength = #nameList[x]
		for i, v in string.gmatch(nameList[x], "[^a-zA-Z]") do
			nameLength = nameLength - 0.5
		end
		local name = nameList[x]..string.rep(" ", 12 - nameLength)
		
		appendText = "|cFF"..F3PAddonGlobalVars.queueData[nameList[x]]["class"]..name.."|cFFFFFFFF | "..wState.."\124cFFFFFFFF | "..aState.."\n"
		outputText = outputText..appendText
	end
	
	F3PQText:SetText(outputText)
end


